rollout fitPrimitives "Fit Primitives"
(

	group "Fit a Box"
	(
		button fitBoxButton "Box.."
	)
	
	group "Fit a Cylinder"
	(
		spinner numSides "Num of Sides" type:#integer range:[4,200,18] 
		button fitCylinderButton "Cylinder.."
	)
	
	group "Fit a Sphere"
	(
		button fitSphereButton "Sphere.."
	)
	
	group "Export"
	(
		button exportPointsButton "Output raw points.."
	)
	
	fn fitSelection numOfSides =
	(
		currentPath = getFilenamePath (getSourceFileName())
		MinOBB_exe = currentPath + "MinOBB.exe"
		MinBall_exe = currentPath + "MinBall.exe"
			
		fileName = currentPath + "points.xyz"
		fitTempFile = currentPath + "result.fit"

		--select $*
		SelectionList = getCurrentSelection()

		for i=1 to SelectionList.count do
		(
			obj = SelectionList[i]
			
			-- Save object points to file
			(
				fs = createFile fileName

				tmesh = snapshotAsMesh obj
				num_verts = tmesh.numverts
				
				for v = 1 to num_verts do
				(
					vert = getVert tmesh v
					format "% % % \n" vert.x vert.y vert.z to:fs
				)
				
				close fs
			)
			
			-- Run MinOBB fitting on saved file
			(
				if numOfSides != 0 then 
					fitCmd = MinOBB_exe + " " + fileName + " > " + fitTempFile
				else
					fitCmd = MinBall_exe + " " + fileName + " > " + fitTempFile
				
				print fitCmd
				DOSCommand fitCmd
			)
			
			-- Construct from fit file
			(
				fs = openFile fitTempFile
				
				try
				(
					-- Fit sphere case
					if numOfSides == 0 then 
					(
						lineText = readLine fs 
						data = filterString lineText " "
						r = data[1] as float
						
						-- Center of sphere
						lineText = readLine fs 
						data = filterString lineText " "

						x = data[1] as float
						y = data[2] as float
						z = data[3] as float		

						mysphere = geosphere radius:r
						mysphere.center = [x, y, z]
						
					)
					else
					(
						-- Minimum of bounding box
						lineText = readLine fs 
						data = filterString lineText " "

						x = data[1] as float
						y = data[2] as float
						z = data[3] as float		
						min_v = [x, y, z]
						
						-- Maximum of bounding box
						lineText = readLine fs 
						data = filterString lineText " "

						x = data[1] as float
						y = data[2] as float
						z = data[3] as float		
						max_v = [x, y, z]
						
						-- Rotation
						lineText = readLine fs 
						data = filterString lineText " "

						x = data[1] as float
						y = data[2] as float
						z = data[3] as float		
						w = data[4] as float
						
						q = quat x y z w

						-- construct primitive
						bbWidth = max_v.x - min_v.x;
						bbLength = max_v.y - min_v.y;
						bbHeight = max_v.z - min_v.z;
						
						format "bbWidth %\n" bbWidth
						format "bbLength %\n" bbLength
						format "bbHeight %\n" bbHeight
						
						centerX = min_v.x + (bbWidth * 0.5);
						centerY = min_v.y + (bbLength * 0.5);
						centerZ = min_v.z + (bbHeight * 0.5);
						
						if numOfSides == 4 then
						(
							mybox = box length:bbLength width:bbWidth height:bbHeight
							mybox.center = [centerX, centerY, centerZ]
							mybox.rotation = q
						)
						else
						(
							cylinderHeight = (amax #(bbWidth, bbLength, bbHeight))
							r = (amin #(bbWidth, bbLength)) * 0.5
							
							mycylinder = cylinder radius:r height:cylinderHeight sides:numOfSides
								
							mycylinder.center = [centerX, centerY, centerZ]
							mycylinder.rotation = q
							mycylinder.pivot = mycylinder.center
							
							select mycylinder
							
							if cylinderHeight == bbWidth then in coordsys local rotate mycylinder (angleaxis 90 [0,1,0])
							if cylinderHeight == bbLength then in coordsys local rotate mycylinder (angleaxis 90 [1,0,0])
							
							format "Cylinder Height %\n" cylinderHeight
						)
					)
					
					close fs
					DOSCommand ("del.exe " + fileName)
				)
				catch 
				(
					close fs
					format "*** % ***\n" (getCurrentException())
				)
			)
			
			redrawViews()
		)
		
		-- Delete temp files
		DOSCommand ("del.exe " + fitTempFile)
		
		--try destroyDialog fitPrimitives catch()
	)
	
	on fitBoxButton pressed do 
	(
		fitSelection(4)
	)
	
	on fitCylinderButton pressed do 
	(
		fitSelection(numSides.value)
	)
	
	on fitSphereButton pressed do 
	(
		fitSelection(0)
	)
	
	on exportPointsButton pressed do
	(
		currentPath = getFilenamePath (getSourceFileName())
			
		fs = createFile (currentPath + "points.xyz")

		try
		(
			tmesh = snapshotAsMesh $
			num_verts = tmesh.numverts
			
			for v = 1 to num_verts do
			(
				vert = getVert tmesh v
				format "% % % \n" vert.x vert.y vert.z to:fs
			)
		)
		catch 
		(
			format "*** % ***\n" (getCurrentException())
		)
		
		close fs
	)
)

try destroyDialog fitPrimitives catch()
createDialog fitPrimitives